@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2010-2012 Jeremy Lainé
- * Contact: http://code.google.com/p/qdjango/
+ * Copyright (C) 2010-2014 Jeremy Lainé
+ * Contact: https://github.com/jlaine/qdjango
  *
  * This file is part of the QDjango Library.
  *
@@ -25,13 +25,6 @@ class QSqlDatabase;
 class QSqlQuery;
 class QString;
 
-/** \defgroup Database
- *
- * QDjango's Object Relation Mapper (ORM) strives to be both powerful
- * and simple to use. Where possible it tries to follow django's
- * ORM API, with a similar lazy queryset mechanism.
- */
-
 /** \brief The QDjango class provides a set of static functions.
  *
  *  It is used to access registered QDjangoModel classes.
@@ -54,9 +47,6 @@ public:
     static QDjangoMetaModel registerModel();
 
 private:
-    // backend specific
-    static QString noLimitSql();
-
     static QDjangoMetaModel registerModel(const QMetaObject *meta);
     static QDjangoMetaModel metaModel(const char *name);
 
