@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2010-2013 Jeremy Lainé
- * Contact: http://code.google.com/p/qdjango/
+ * Copyright (C) 2010-2014 Jeremy Lainé
+ * Contact: https://github.com/jlaine/qdjango
  *
  * This file is part of the QDjango Library.
  *
@@ -58,7 +58,7 @@ static long stringlist_hash(const QStringList &l)
 
 static QString stringlist_digest(const QStringList &l)
 {
-    return QString::number(labs(stringlist_hash(l)) % 4294967296L, 16);
+    return QString::number(labs(stringlist_hash(l)) % 4294967296UL, 16);
 }
 
 enum ForeignKeyConstraint
@@ -231,6 +231,7 @@ static bool stringToBool(const QString &value)
 class QDjangoMetaModelPrivate : public QSharedData
 {
 public:
+    QString className;
     QList<QDjangoMetaField> localFields;
     QMap<QByteArray, QByteArray> foreignFields;
     QByteArray primaryKey;
@@ -247,6 +248,7 @@ QDjangoMetaModel::QDjangoMetaModel(const QMetaObject *meta)
     if (!meta)
         return;
 
+    d->className = meta->className();
     d->table = QString::fromLatin1(meta->className()).toLower();
 
     // parse table options
@@ -375,7 +377,7 @@ QDjangoMetaModel::QDjangoMetaModel(const QMetaObject *meta)
         d->localFields.prepend(field);
         d->primaryKey = field.d->name;
     }
- 
+
 }
 
 /*!
@@ -393,6 +395,19 @@ QDjangoMetaModel::~QDjangoMetaModel()
 {
 }
 
+QString QDjangoMetaModel::className() const
+{
+    return d->className;
+}
+
+/*!
+    Determine whether this is a valid model, or just default constructed
+ */
+bool QDjangoMetaModel::isValid() const
+{
+    return !d->table.isNull();
+}
+
 /*!
     Assigns \a other to this meta model.
 */
@@ -423,25 +438,34 @@ QStringList QDjangoMetaModel::createTableSql() const
 {
     QSqlDatabase db = QDjango::database();
     QSqlDriver *driver = db.driver();
-    const QString driverName = db.driverName();
+    QDjangoDatabase::DatabaseType databaseType = QDjangoDatabase::databaseType(db);
 
     QStringList queries;
     QStringList propSql;
+    QStringList constraintSql;
     const QString quotedTable = db.driver()->escapeIdentifier(d->table, QSqlDriver::TableName);
     foreach (const QDjangoMetaField &field, d->localFields)
     {
         QString fieldSql = driver->escapeIdentifier(field.column(), QSqlDriver::FieldName);
         switch (field.d->type) {
         case QVariant::Bool:
-            if (driverName == QLatin1String("QPSQL"))
+            if (databaseType == QDjangoDatabase::PostgreSQL)
                 fieldSql += QLatin1String(" boolean");
+            else if (databaseType == QDjangoDatabase::MSSqlServer)
+                fieldSql += QLatin1String(" bit");
             else
                 fieldSql += QLatin1String(" bool");
             break;
         case QVariant::ByteArray:
-            if (driverName == QLatin1String("QPSQL"))
+            if (databaseType == QDjangoDatabase::PostgreSQL) {
                 fieldSql += QLatin1String(" bytea");
-            else {
+            } else if (databaseType == QDjangoDatabase::MSSqlServer) {
+                fieldSql += QLatin1String(" varbinary");
+                if (field.d->maxLength > 0)
+                    fieldSql += QLatin1Char('(') + QString::number(field.d->maxLength) + QLatin1Char(')');
+                else
+                    fieldSql += QLatin1String("(max)");
+            } else {
                 fieldSql += QLatin1String(" blob");
                 if (field.d->maxLength > 0)
                     fieldSql += QLatin1Char('(') + QString::number(field.d->maxLength) + QLatin1Char(')');
@@ -451,7 +475,7 @@ QStringList QDjangoMetaModel::createTableSql() const
             fieldSql += QLatin1String(" date");
             break;
         case QVariant::DateTime:
-            if (driverName == QLatin1String("QPSQL"))
+            if (databaseType == QDjangoDatabase::PostgreSQL)
                 fieldSql += QLatin1String(" timestamp");
             else
                 fieldSql += QLatin1String(" datetime");
@@ -460,16 +484,26 @@ QStringList QDjangoMetaModel::createTableSql() const
             fieldSql += QLatin1String(" real");
             break;
         case QVariant::Int:
-            fieldSql += QLatin1String(" integer");
+            if (databaseType == QDjangoDatabase::MSSqlServer)
+                fieldSql += QLatin1String(" int");
+            else
+                fieldSql += QLatin1String(" integer");
             break;
         case QVariant::LongLong:
             fieldSql += QLatin1String(" bigint");
             break;
         case QVariant::String:
-            if (field.d->maxLength > 0)
-                fieldSql += QLatin1String(" varchar(") + QString::number(field.d->maxLength) + QLatin1Char(')');
-            else
-                fieldSql += QLatin1String(" text");
+            if (field.d->maxLength > 0) {
+                if (databaseType == QDjangoDatabase::MSSqlServer)
+                    fieldSql += QLatin1String(" nvarchar(") + QString::number(field.d->maxLength) + QLatin1Char(')');
+                else
+                    fieldSql += QLatin1String(" varchar(") + QString::number(field.d->maxLength) + QLatin1Char(')');
+            } else {
+                if (databaseType == QDjangoDatabase::MSSqlServer)
+                    fieldSql += QLatin1String(" nvarchar(max)");
+                else
+                    fieldSql += QLatin1String(" text");
+            }
             break;
         case QVariant::Time:
             fieldSql += QLatin1String(" time");
@@ -490,16 +524,17 @@ QStringList QDjangoMetaModel::createTableSql() const
 
         // auto-increment is backend specific
         if (field.d->autoIncrement) {
-            if (driverName == QLatin1String("QSQLITE") ||
-                driverName == QLatin1String("QSQLITE2"))
+            if (databaseType == QDjangoDatabase::SQLite)
                 // NOTE: django does not add this option for sqlite, but there
                 // is a ticket asking for it to do so:
                 // https://code.djangoproject.com/ticket/10164
                 fieldSql += QLatin1String(" AUTOINCREMENT");
-            else if (driverName == QLatin1String("QMYSQL"))
+            else if (databaseType == QDjangoDatabase::MySqlServer)
                 fieldSql += QLatin1String(" AUTO_INCREMENT");
-            else if (driverName == QLatin1String("QPSQL"))
+            else if (databaseType == QDjangoDatabase::PostgreSQL)
                 fieldSql = driver->escapeIdentifier(field.column(), QSqlDriver::FieldName) + QLatin1String(" serial PRIMARY KEY");
+            else if (databaseType == QDjangoDatabase::MSSqlServer)
+                fieldSql += QLatin1String(" IDENTITY(1,1)");
         }
 
         // foreign key
@@ -507,33 +542,74 @@ QStringList QDjangoMetaModel::createTableSql() const
         {
             const QDjangoMetaModel foreignMeta = QDjango::metaModel(field.d->foreignModel);
             const QDjangoMetaField foreignField = foreignMeta.localField("pk");
-            fieldSql += QString::fromLatin1(" REFERENCES %1 (%2)").arg(
-                driver->escapeIdentifier(foreignMeta.d->table, QSqlDriver::TableName),
-                driver->escapeIdentifier(foreignField.column(), QSqlDriver::FieldName));
+            if (databaseType == QDjangoDatabase::MySqlServer) {
+                QString constraintName = QString::fromLatin1("FK_%1_%2").arg(
+                    field.column(), stringlist_digest(QStringList() << field.column() << d->table));
+                QString constraint =
+                    QString::fromLatin1("CONSTRAINT %1 FOREIGN KEY (%2) REFERENCES %3 (%4)").arg(
+                        driver->escapeIdentifier(constraintName, QSqlDriver::FieldName),
+                        driver->escapeIdentifier(field.column(), QSqlDriver::FieldName),
+                        driver->escapeIdentifier(foreignMeta.d->table, QSqlDriver::TableName),
+                        driver->escapeIdentifier(foreignField.column(), QSqlDriver::FieldName)
+                    );
 
-            if (field.d->deleteConstraint != NoAction) {
-                fieldSql += " ON DELETE";
-                switch (field.d->deleteConstraint) {
-                case Cascade:
-                    fieldSql += " CASCADE";
-                    break;
-                case SetNull:
-                    fieldSql += " SET NULL";
-                    break;
-                case Restrict:
-                    fieldSql += " RESTRICT";
-                    break;
-                default:
+                if (field.d->deleteConstraint != NoAction) {
+                    constraint += " ON DELETE";
+                    switch (field.d->deleteConstraint) {
+                    case Cascade:
+                        constraint += " CASCADE";
+                        break;
+                    case SetNull:
+                        constraint += " SET NULL";
+                        break;
+                    case Restrict:
+                        constraint += " RESTRICT";
+                        break;
+                    default:
+                        break;
+                    }
+                }
+
+                constraintSql << constraint;
+            } else {
+                fieldSql += QString::fromLatin1(" REFERENCES %1 (%2)").arg(
+                    driver->escapeIdentifier(foreignMeta.d->table, QSqlDriver::TableName),
+                    driver->escapeIdentifier(foreignField.column(), QSqlDriver::FieldName));
+
+                if (databaseType == QDjangoDatabase::MSSqlServer &&
+                    field.d->deleteConstraint == Restrict) {
+                    qWarning("MSSQL does not support RESTRICT constraints");
                     break;
                 }
+
+                if (field.d->deleteConstraint != NoAction) {
+                    fieldSql += " ON DELETE";
+                    switch (field.d->deleteConstraint) {
+                    case Cascade:
+                        fieldSql += " CASCADE";
+                        break;
+                    case SetNull:
+                        fieldSql += " SET NULL";
+                        break;
+                    case Restrict:
+                        fieldSql += " RESTRICT";
+                        break;
+                    default:
+                        break;
+                    }
+                }
             }
 
-            if (driverName == QLatin1String("QPSQL"))
+            if (databaseType == QDjangoDatabase::PostgreSQL)
                 fieldSql += " DEFERRABLE INITIALLY DEFERRED";
         }
         propSql << fieldSql;
     }
 
+    // add constraints if we need them
+    if (!constraintSql.isEmpty())
+        propSql << constraintSql.join(QLatin1String(", "));
+
     // unique contraints
     if (!d->uniqueTogether.isEmpty()) {
         QStringList columns;
