@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2010-2013 Jeremy Lainé
+ * Copyright (C) 2010-2014 Jeremy Lainé
  * Copyright (C) 2011 Mathias Hasselmann
- * Contact: http://code.google.com/p/qdjango/
+ * Contact: https://github.com/jlaine/qdjango
  *
  * This file is part of the QDjango Library.
  *
@@ -16,9 +16,12 @@
  * Lesser General Public License for more details.
  */
 
+#include <QDebug>
 #include <QSqlDriver>
+#include <QSqlRecord>
 
 #include "QDjango.h"
+#include "QDjango_p.h"
 #include "QDjangoQuerySet.h"
 #include "QDjangoWhere_p.h"
 
@@ -30,38 +33,63 @@ QDjangoCompiler::QDjangoCompiler(const char *modelName, const QSqlDatabase &db)
     baseModel = QDjango::metaModel(modelName);
 }
 
-QString QDjangoCompiler::referenceModel(const QString &modelPath, QDjangoMetaModel *metaModel)
+QString QDjangoCompiler::referenceModel(const QString &modelPath, QDjangoMetaModel *metaModel, bool nullable)
 {
     if (modelPath.isEmpty())
         return driver->escapeIdentifier(baseModel.table(), QSqlDriver::TableName);
 
     if (modelRefs.contains(modelPath))
-        return modelRefs.value(modelPath).first;
+        return modelRefs.value(modelPath).tableReference;
 
     const QString modelRef = QLatin1String("T") + QString::number(modelRefs.size());
-    modelRefs.insert(modelPath, qMakePair(modelRef, *metaModel));
+    modelRefs.insert(modelPath, QDjangoModelReference(modelRef, *metaModel, nullable));
     return modelRef;
 }
 
 QString QDjangoCompiler::databaseColumn(const QString &name)
 {
     QDjangoMetaModel model = baseModel;
+    QString modelName;
     QString modelPath;
-    QString modelRef = referenceModel(QString(), &model);
-
+    QString modelRef = referenceModel(QString(), &model, false);
     QStringList bits = name.split(QLatin1String("__"));
+
     while (bits.size() > 1) {
         const QByteArray fk = bits.first().toLatin1();
-        if (!model.foreignFields().contains(fk))
-            break;
+        QDjangoMetaModel foreignModel;
+        bool foreignNullable = false;
 
-        QDjangoMetaModel foreignModel = QDjango::metaModel(model.foreignFields()[fk]);
-
-        // store reference
         if (!modelPath.isEmpty())
             modelPath += QLatin1String("__");
         modelPath += bits.first();
-        modelRef = referenceModel(modelPath, &foreignModel);
+
+        if (!model.foreignFields().contains(fk)) {
+            // this might be a reverse relation, so look for the model
+            // and if it exists continue
+            foreignModel = QDjango::metaModel(fk);
+            QDjangoReverseReference rev;
+            const QMap<QByteArray, QByteArray> foreignFields = foreignModel.foreignFields();
+            foreach (const QByteArray &foreignKey, foreignFields.keys()) {
+                if (foreignFields[foreignKey] == baseModel.className()) {
+                    rev.leftHandKey = foreignModel.localField(foreignKey + "_id").column();
+                    break;
+                }
+            }
+
+            if (rev.leftHandKey.isEmpty()) {
+                qWarning() << "Invalid field lookup" << name;
+                return QString();
+            }
+            rev.rightHandKey = foreignModel.primaryKey();
+            reverseModelRefs[modelPath] = rev;
+        } else {
+            foreignModel = QDjango::metaModel(model.foreignFields()[fk]);
+            foreignNullable = model.localField(fk + QByteArray("_id")).isNullable();;
+        }
+
+        // store reference
+        modelRef = referenceModel(modelPath, &foreignModel, foreignNullable);
+        modelName = fk;
 
         model = foreignModel;
         bits.takeFirst();
@@ -71,14 +99,14 @@ QString QDjangoCompiler::databaseColumn(const QString &name)
     return modelRef + QLatin1Char('.') + driver->escapeIdentifier(field.column(), QSqlDriver::FieldName);
 }
 
-QStringList QDjangoCompiler::fieldNames(bool recurse, QDjangoMetaModel *metaModel, const QString &modelPath)
+QStringList QDjangoCompiler::fieldNames(bool recurse, QDjangoMetaModel *metaModel, const QString &modelPath, bool nullable)
 {
     QStringList columns;
     if (!metaModel)
         metaModel = &baseModel;
 
     // store reference
-    const QString tableName = referenceModel(modelPath, metaModel);
+    const QString tableName = referenceModel(modelPath, metaModel, nullable);
     foreach (const QDjangoMetaField &field, metaModel->localFields())
         columns << tableName + QLatin1Char('.') + driver->escapeIdentifier(field.column(), QSqlDriver::FieldName);
     if (!recurse)
@@ -88,7 +116,8 @@ QStringList QDjangoCompiler::fieldNames(bool recurse, QDjangoMetaModel *metaMode
     const QString pathPrefix = modelPath.isEmpty() ? QString() : (modelPath + QLatin1String("__"));
     foreach (const QByteArray &fkName, metaModel->foreignFields().keys()) {
         QDjangoMetaModel metaForeign = QDjango::metaModel(metaModel->foreignFields()[fkName]);
-        columns += fieldNames(recurse, &metaForeign, pathPrefix + QString::fromLatin1(fkName));
+        bool nullableForeign = metaModel->localField(fkName + QByteArray("_id")).isNullable();
+        columns += fieldNames(recurse, &metaForeign, pathPrefix + QString::fromLatin1(fkName), nullableForeign);
     }
     return columns;
 }
@@ -97,19 +126,28 @@ QString QDjangoCompiler::fromSql()
 {
     QString from = driver->escapeIdentifier(baseModel.table(), QSqlDriver::TableName);
     foreach (const QString &name, modelRefs.keys()) {
-        baseModel.localField(name.toLatin1() + QByteArray("_id"));
-        from += QString::fromLatin1(" %1 %2 %3 ON %4.%5 = %6")
-            .arg(baseModel.localField(name.toLatin1() + QByteArray("_id")).isNullable() ? "LEFT OUTER JOIN" : "INNER JOIN")
-            .arg(driver->escapeIdentifier(modelRefs[name].second.table(), QSqlDriver::TableName))
-            .arg(modelRefs[name].first)
-            .arg(modelRefs[name].first)
-            .arg(driver->escapeIdentifier(modelRefs[name].second.localField("pk").column(), QSqlDriver::FieldName))
-            .arg(databaseColumn(name + QLatin1String("_id")));
+        const QDjangoModelReference &ref = modelRefs[name];
+
+        QString leftHandColumn, rightHandColumn;
+        if (reverseModelRefs.contains(name)) {
+            const QDjangoReverseReference &rev = reverseModelRefs[name];
+            leftHandColumn = ref.tableReference + "." + driver->escapeIdentifier(rev.leftHandKey, QSqlDriver::FieldName);;
+            rightHandColumn = databaseColumn(rev.rightHandKey);
+        } else {
+            leftHandColumn = databaseColumn(name + QLatin1String("__pk"));
+            rightHandColumn = databaseColumn(name + QLatin1String("_id"));
+        }
+        from += QString::fromLatin1(" %1 %2 %3 ON %4 = %5")
+            .arg(ref.nullable ? "LEFT OUTER JOIN" : "INNER JOIN")
+            .arg(driver->escapeIdentifier(ref.metaModel.table(), QSqlDriver::TableName))
+            .arg(ref.tableReference)
+            .arg(leftHandColumn)
+            .arg(rightHandColumn);
     }
     return from;
 }
 
-QString QDjangoCompiler::orderLimitSql(const QStringList orderBy, int lowMark, int highMark)
+QString QDjangoCompiler::orderLimitSql(const QStringList &orderBy, int lowMark, int highMark)
 {
     QString limit;
 
@@ -126,19 +164,43 @@ QString QDjangoCompiler::orderLimitSql(const QStringList orderBy, int lowMark, i
         }
         bits.append(databaseColumn(field) + QLatin1Char(' ') + order);
     }
+
     if (!bits.isEmpty())
         limit += QLatin1String(" ORDER BY ") + bits.join(QLatin1String(", "));
 
     // limits
-    if (highMark > 0)
-        limit += QLatin1String(" LIMIT ") + QString::number(highMark - lowMark);
-    if (lowMark > 0)
-    {
-        // no-limit is backend specific
-        if (highMark <= 0)
-            limit += QDjango::noLimitSql();
-        limit += QLatin1String(" OFFSET ") + QString::number(lowMark);
+    QDjangoDatabase::DatabaseType databaseType =
+        QDjangoDatabase::databaseType(QDjango::database());
+
+    if (databaseType == QDjangoDatabase::MSSqlServer) {
+        if (limit.isEmpty() && (highMark > 0 || lowMark > 0))
+            limit += QLatin1String(" ORDER BY ") + databaseColumn(baseModel.primaryKey());
+
+        if (lowMark > 0 || (lowMark == 0 && highMark > 0)) {
+            limit += QLatin1String(" OFFSET ") + QString::number(lowMark);
+            limit += QLatin1String(" ROWS");
+        }
+
+        if (highMark > 0)
+            limit += QString(" FETCH NEXT %1 ROWS ONLY").arg(highMark - lowMark);
+    } else {
+        if (highMark > 0)
+            limit += QLatin1String(" LIMIT ") + QString::number(highMark - lowMark);
+
+        if (lowMark > 0) {
+            // no-limit is backend specific
+            if (highMark <= 0) {
+                if (databaseType == QDjangoDatabase::SQLite)
+                    limit += QLatin1String(" LIMIT -1");
+                else if (databaseType == QDjangoDatabase::MySqlServer)
+                    // 2^64 - 1, as recommended by the MySQL documentation
+                    limit += QLatin1String(" LIMIT 18446744073709551615");
+            }
+
+            limit += QLatin1String(" OFFSET ") + QString::number(lowMark);
+        }
     }
+
     return limit;
 }
 
@@ -209,32 +271,16 @@ bool QDjangoQuerySetPrivate::sqlFetch()
     if (hasResults || whereClause.isNone())
         return true;
 
-    QSqlDatabase db = QDjango::database();
-
-    // build query
-    QDjangoCompiler compiler(m_modelName, db);
-    QDjangoWhere resolvedWhere(whereClause);
-    compiler.resolve(resolvedWhere);
-
-    const QStringList columns = compiler.fieldNames(selectRelated);
-    const QString where = resolvedWhere.sql(db);
-    const QString limit = compiler.orderLimitSql(orderBy, lowMark, highMark);
-    QString sql = QLatin1String("SELECT ") + columns.join(QLatin1String(", ")) + QLatin1String(" FROM ") + compiler.fromSql();
-    if (!where.isEmpty())
-        sql += QLatin1String(" WHERE ") + where;
-    sql += limit;
-    QDjangoQuery query(db);
-    query.prepare(sql);
-    resolvedWhere.bindValues(query);
-
     // execute query
+    QDjangoQuery query(selectQuery());
     if (!query.exec())
         return false;
 
     // store results
     while (query.next()) {
         QVariantList props;
-        for (int i = 0; i < columns.size(); ++i)
+        const int propCount = query.record().count();
+        for (int i = 0; i < propCount; ++i)
             props << query.value(i);
         properties.append(props);
     }
@@ -252,7 +298,9 @@ bool QDjangoQuerySetPrivate::sqlInsert(const QVariantMap &fields, QVariant *inse
     // fetch autoincrement pk
     if (insertId) {
         QSqlDatabase db = QDjango::database();
-        if (db.driverName() == QLatin1String("QPSQL")) {
+        QDjangoDatabase::DatabaseType databaseType = QDjangoDatabase::databaseType(db);
+
+        if (databaseType == QDjangoDatabase::PostgreSQL) {
             const QDjangoMetaModel metaModel = QDjango::metaModel(m_modelName);
             QDjangoQuery query(db);
             const QDjangoMetaField primaryKey = metaModel.localField("pk");
@@ -364,6 +412,30 @@ QDjangoQuery QDjangoQuerySetPrivate::insertQuery(const QVariantMap &fields) cons
     return query;
 }
 
+/** Returns the SQL query to perform a SELECT on the current set.
+ */
+QDjangoQuery QDjangoQuerySetPrivate::selectQuery() const
+{
+    QSqlDatabase db = QDjango::database();
+
+    // build query
+    QDjangoCompiler compiler(m_modelName, db);
+    QDjangoWhere resolvedWhere(whereClause);
+    compiler.resolve(resolvedWhere);
+
+    const QStringList columns = compiler.fieldNames(selectRelated);
+    const QString where = resolvedWhere.sql(db);
+    const QString limit = compiler.orderLimitSql(orderBy, lowMark, highMark);
+    QString sql = QLatin1String("SELECT ") + columns.join(QLatin1String(", ")) + QLatin1String(" FROM ") + compiler.fromSql();
+    if (!where.isEmpty())
+        sql += QLatin1String(" WHERE ") + where;
+    sql += limit;
+    QDjangoQuery query(db);
+    query.prepare(sql);
+    resolvedWhere.bindValues(query);
+    return query;
+}
+
 /** Returns the SQL query to perform an UPDATE on the current set for the
     specified \a fields.
  */
