@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2010-2012 Jeremy Lainé
- * Contact: http://code.google.com/p/qdjango/
+ * Copyright (C) 2010-2014 Jeremy Lainé
+ * Contact: https://github.com/jlaine/qdjango
  *
  * This file is part of the QDjango Library.
  *
@@ -32,12 +32,15 @@
 class QDjangoWherePrivate : public QSharedData
 {
 public:
+    static QString operationToString(QDjangoWhere::Operation operation);
+
     enum Combine
     {
         NoCombine,
         AndCombine,
-        OrCombine,
+        OrCombine
     };
+    static QString combineToString(Combine combine);
 
     QDjangoWherePrivate();
 
