@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2010-2012 Jeremy Lainé
- * Contact: http://code.google.com/p/qdjango/
+ * Copyright (C) 2010-2014 Jeremy Lainé
+ * Contact: https://github.com/jlaine/qdjango
  *
  * This file is part of the QDjango Library.
  *
@@ -179,8 +179,6 @@ QDjangoHttpResponse *QDjangoHttpController::serveStatic(const QDjangoHttpRequest
         mimeType = QLatin1String("text/html");
     else if (fileName.endsWith(QLatin1String(".js")))
         mimeType = QLatin1String("application/javascript");
-    else if (fileName.endsWith(QLatin1String(".png")))
-        mimeType = QLatin1String("image/png");
     else
         mimeType = QLatin1String("application/octet-stream");
     response->setHeader(QLatin1String("Content-Type"), mimeType);
