@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2010-2012 Jeremy Lainé
- * Contact: http://code.google.com/p/qdjango/
+ * Copyright (C) 2010-2014 Jeremy Lainé
+ * Contact: https://github.com/jlaine/qdjango
  *
  * This file is part of the QDjango Library.
  *
@@ -21,11 +21,14 @@
 #if defined(QDJANGO_SHARED)
 #  if defined(QDJANGO_BUILD)
 #    define QDJANGO_EXPORT Q_DECL_EXPORT
+#    define QDJANGO_AUTOTEST_EXPORT Q_DECL_EXPORT
 #  else
 #    define QDJANGO_EXPORT Q_DECL_IMPORT
+#    define QDJANGO_AUTOTEST_EXPORT Q_DECL_IMPORT
 #  endif
 #else
 #  define QDJANGO_EXPORT
+#  define QDJANGO_AUTOTEST_EXPORT
 #endif
 
 #endif
